! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! In this example, a dataset is created with one table and one
! array.
! Two attributes are added to each of the dataset, table and array.
!
! The generic subroutine displayAttributes, which operates on the
! AttributableT base type, displays the attributes contained in
! each of the dataset, table and array.
subroutine displayAttributes( thisAttributable )
  use dal

  implicit none

  type(AttributableT) thisAttributable
  type(AttributeT) att
  integer i

  do i = 0, numberOfAttributes( thisAttributable ) - 1
    att = attribute( thisAttributable, i )
    write(*,*) name( att ), stringAttribute( att ), units( att ), label( att )
  end do

end subroutine displayAttributes

program example_attributable
   
  use dal

  implicit none
   
  type(DataSetT) set
  type(TableT) tab
  type(ArrayT) arr
  integer(kind=int32), dimension(:,:,:), pointer :: a
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)

  set = dataSet("test.dat",CREATE)
  call setAttribute(set,"sbool1",.false.,"dataset first bool comment")
  call setAttribute(set,"sbool2",.true.,"dataset second comment")
   
  tab = addTable(set,"table",10);
  call setAttribute(set,"int1",1,"table first integer comment","kg")
  call setAttribute(set,"int2",2,"table second integer comment","mm")

  arr = addArray(set, "array", INTEGER32, dimensions=s )
  call setAttribute(set,"real1",1.1,"array first real comment","kN")
  call setAttribute(set,"real2",2.3,"array second real comment","rad")

  call displayAttributes( attributable( set ) )
  call displayAttributes( attributable( tab ) )
  call displayAttributes( attributable( arr ) )

  call release(set)
   
end program example_attributable
