! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! In this example add dataset is created containing
! 2 arrays and 2 tables.
!
! A simple loop then iterates over the dataset's
! blocks printing appropriate messages.
! The first two blocks will have ARRAY_BLOCK block type.
! The second two blocks will have TABLE_BLOCK block type.
program example_blocktype

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(ArrayT) arr
  type(TableT) tab
  type(BlockT) blk
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)
  integer :: i

  ! create a set
  set = dataSet("test.dat",CREATE)
  arr = addArray(set, "block0", INTEGER32, dimensions=s )
  arr = addArray(set, "block1", INTEGER32, dimensions=s )
  tab = addTable(set, "block2", 5 )
  tab = addTable(set, "block3", 5 )

  do i = 0, numberOfBlocks( set ) - 1
    blk = block( set, i, READ )

    if( blockType( block( set, i, READ ) ) .eq. ARRAY_BLOCK ) then 
      write(*,*) "The block with name ", name( blk ), " is an array."
      arr = array( set, i, READ )
      write(*,*) "It has ", numberOfDimensions( arr ), " dimensions."
    end if

    if( blockType( block( set, i, READ ) ) .eq. TABLE_BLOCK ) then 
      write(*,*) "The block with name ", name( blk ), " is a table."
      tab = table( set, i )
      write(*,*) "It has ", numberOfRows( tab ), " rows."
    end if
  end do

  call release(arr)
  call release(set)

end program example_blocktype

