! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! In this example add dataset is created (opened) containing
! a table with 2 columns of two 4-dimensional arrays.
!
! The second array has the same data type as the first; this
! is ensured by using the arrayDataType() function to determine
! the data type of the first array.
!
! The columns are then  initialised, and then the second column
! is output by accessing the column's data as a flat vector.
program example_booldata

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col1, col2
  logical(kind=BOOL), dimension(:,:,:,:,:), pointer :: c1, c2
  logical(kind=BOOL), dimension(:), pointer :: cd
  integer, dimension(4), parameter :: s = (/ 3,4,5,6 /)
  integer :: i,j,k,l,m,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  tab = addTable(set, "table", 5, "table comment" )
  col1 = addColumn( tab, "column1", BOOLEAN, "km", s, "column comment" )
  col2 = addColumn( tab, "column2", columnDataType( col1 ), "km", s, "column comment" )

  ! fill with unique numbers
  c1 => boolArray4Data(col1)
  c2 => boolArray4Data(col2)

  n = 0
  do m=0,numberOfRows(tab) - 1
    do l=0,5
      do k=0,4
        do j=0,3
          do i=0,2 
            c1(i,j,k,l,m) = .false.
            c2(i,j,k,l,m) = c1(i,j,k,l,m)
            n = n + 1
          end do
        end do
      end do
    end do
  end do

  call release(col1)
  call release(col2)

  ! Output the col2
  cd => boolData( col2 ) ! Access the column's 4-dimensional data as a flat vector.

  do n = 0,numberOfElements(col1) * numberOfRows(tab) - 1
    write(*,*) cd(n)
  end do

  call release(col2)
  call release(set)

end program example_booldata
