! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! This example showas how the hasBlock() function is used.
! In the example, a dataset is created with one table and one
! array.
! The generic subroutine displayBlock, which operates on the
! BlockT base type. The blockType() function operates on objects
! of type BlockT.
! The dataset is testes for the existence of the table and the array, and in
! each case, the block is displayed.
subroutine displayBlock( thisBlock )
  use dal

  implicit none

  type(BlockT) thisBlock

  write(*,*) "The block with name ", name( thisBlock )

  if( blockType( thisBlock ) .eq. ARRAY_BLOCK ) then
    write(*,*) " is an array."
  end if

  if( blockType( thisBlock ) .eq. TABLE_BLOCK ) then
    write(*,*) " is a table."
  end if

end subroutine displayBlock

program example_hasblock
   
  use dal

  implicit none
   
  type(DataSetT) set
  type(TableT) tab
  type(ArrayT) arr
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)

  set = dataSet("test.dat",CREATE)
  tab = addTable(set,"table",10);
  arr = addArray(set, "array", INTEGER32, dimensions=s )

  call release(set)

  set = dataSet("test.dat",READ)

  if( hasBlock( set, "table" ) ) then
    call displayBlock( block( set, "table",READ ) )
  end if

  if( hasBlock( set, "array" ) ) then
    call displayBlock( block( set, "array",READ ) )
  end if

  call release(set)
   
end program example_hasblock
