! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! In this example add dataset is created (opened) containing
! a table with 2 columns of two vector arrays.
!
! The second column has the same data type as the first; this
! is ensured by using the columnDataType() function to determine
! the data type of the first array.
!
! The columns are then  initialised before the
! dataset is released (closed).
program example_columnvectordata

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col1, col2
  integer(kind=INT16), dimension(:,:), pointer :: c1, c2
  integer, dimension(1), parameter :: s = (/ 3 /)
  integer :: i,m,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  tab = addTable(set, "table", 10, "table comment" )
  col1 = addColumn( tab, "column1", INTEGER16, "km", s, "column comment" )
  col2 = addColumn( tab, "column2", columnDataType( col1 ), "km", s, "column comment" )

  ! fill with unique numbers
  c1 => int16VectorData(col1)
  c2 => int16VectorData(col2)

  n = 0
  do m=0,numberOfRows(tab) - 1
    do i=0,2 
      c1(i,m) = n
      c2(i,m) = c1(i,m)
      n = n + 1
    end do
  end do

  call release(col1)
  call release(col2)
  call release(set)

end program example_columnvectordata
