! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! In this example add dataset is created (opened) containing
! a table with 2 columns of two 2-dimensional arrays.
!
! The second array has the same data type as the first; this
! is ensured by using the arrayDataType() function to determine
! the data type of the first array.
!
! The columns are then  initialised before the
! dataset is released (closed).
program example_array2data

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col1, col2
  integer(kind=INT32), dimension(:,:,:), pointer :: c1, c2
  integer, dimension(2), parameter :: s = (/ 3,4 /)
  integer :: i,j,k,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  tab = addTable(set, "table", 100, "table comment" )
  col1 = addColumn( tab, "column1", INTEGER32, "km", s, "column comment" )
  col2 = addColumn( tab, "column2", columnDataType( col1 ), "km", s, "column comment" )

  ! fill with unique numbers
  c1 => int32Array2Data(col1)
  c2 => int32Array2Data(col2)

  n = 0
  do k=0,numberOfRows(tab) - 1
    do j=0,3
      do i=0,2 
        c1(i,j,k) = n
        c2(i,j,k) = c1(i,j,k)
        n = n + 1
      end do
    end do
  end do

  call release(col1)
  call release(col2)
  call release(set)

end program example_array2data
