// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
/*************************************************************** 
* 
* DalDal.cc          & XMM Science Analysis System (SAS) 
* 
* Class representing Dal
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalDal.cc,v 1.2 2004/03/05 14:43:01 eojero Exp $
* 
*****************************************************************/ 


#include <errstr.h>
#include <Tracer.h>
#include "DalDal.h"


DalDal::DalDal( AccessMode accessMode, const DalDal * parentAccessMode )
    :	_accessMode( accessMode ),
	_alwaysAllowModify( parentAccessMode ? parentAccessMode -> _alwaysAllowModify : false ),
		_parentAccessMode( parentAccessMode )
{
    PROC
	ENDPROC
}


Dal::AccessMode
DalDal::mode() const
{
	PROC
	RETURN _accessMode == AsParent ? _parentAccessMode -> mode() : _accessMode;
}


void
DalDal::mode( AccessMode newAccessMode )
{
    PROC
	_accessMode = newAccessMode;
    ENDPROC
}


void
DalDal::alwaysAllowModify( bool onoff )
{
	PROC
	_alwaysAllowModify = onoff;
	ENDPROC
}


void
DalDal::checkIfReadOnly() const
{
	PROC
	    if(
		mode() == Dal::Read
		&&
		!_alwaysAllowModify )
	    {
		errstr << "An attempt was made to modify a read-only object." << Sas::error( "readOnly" );
	    } // if
	ENDPROC
}
