// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* DalVColumnData.h                  & XMM Science Analysis System (SAS) 
* 
* Class representing data object.
* 
* (c) 2001 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalVColumnData.h,v 1.2 2004/03/05 14:43:01 eojero Exp $ 
* 
*****************************************************************/ 
#ifndef DAL_VCOLUMN_DATA_H
#define DAL_VCOLUMN_DATA_H


//#include <string>
//#include <vector>


#include "Dal.h"
#include "Tracer.h"
#include "DalColumnData.h"


class DalVColumnData : public virtual ColumnData
{
	public :

		// Constructors
		DalVColumnData(
			CellData*			cellData );

		virtual ~DalVColumnData();
		AccessMode 		mode() const; 
		int8 * 			int8Data() const; 
		uint16 * 		uint16Data() const; 
		int16 * 		int16Data() const; 
		uint32 * 		uint32Data() const; 
		int32 * 		int32Data() const; 
		real32 * 		real32Data() const; 
		real64 * 		real64Data() const; 
		bool8 * 		boolData() const; 
		char* 			stringData() const; 

    		unsigned long 		position(
						unsigned long	/*row*/) const;

    		unsigned long 		position(
						unsigned long	/*row*/,
						unsigned long 	/*pos*/) const;

    		unsigned long 		position(
						unsigned long	/*row*/,
						const longVec& 	/*pos*/) const;

		void 			release();
		bool 			isNull(long unsigned pos ) const;
		void 			setToNull(long unsigned pos ) const;
		bool 			hasNulls() const;
		unsigned int 		dataSize() const;
		unsigned long 		elements() const;
		const longVec& 		size() const;
		Data::DataType 		dataType() const ;
		void 			printOn( ostream & os, const std::string & sep="" ) const;
		
		void 			seek( unsigned long  from, unsigned long count ); 
		long unsigned int 	from() const; 
		long unsigned int 	count() const; 
		Column *		parent(); 
		const Column * 		parent() const; 
		void 			parent( const Column * parent );
		bool 			isOrphan() const; 

	private :
		CellData*			_data;
};


#endif

