// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//  Giuseppe Vacanti (cosine science & computing bv)
//  September 12, 2002
// 
//   $Id: DataSetRecord.cc,v 1.2 2004/03/05 14:43:02 eojero Exp $

#include<DataSetRecord.h>
#include<errstr.h>

DataSetRecord::DataSetRecord(const std::string & name)
    : name_(name),
      keep_(false),
      set_(0),
      count_(0),
      model_(DataSetServer::UseEnvironment)
{}

void DataSetRecord::dataSet(DataSet * set){
    set_ = set;
    ++(*this);
}

DataSet * DataSetRecord::dataSet() const{
    return set_;
}

Dal::AccessMode DataSetRecord::mode() const{
    if(! active())
	errstr << "The dataset " << name_ << " is not active"
	       << Sas::error("DataSetNonActive");
    return set_->mode();
}

DataSetServer::MemoryModel DataSetRecord::model() const{
    return model_;
}

void DataSetRecord::model(DataSetServer::MemoryModel model){
    model = model_;
}

DataSetRecord & DataSetRecord::operator++(){
    ++count_;
    return *this;
}
 
// DataSetRecord DataSetRecord::operator++(int){
//     DataSetRecord old(*this);
//     ++*this;
//     return old;
// }

DataSetRecord & DataSetRecord::operator--(){
    --count_;
    return *this;
}
 
// DataSetRecord DataSetRecord::operator--(int){
//     DataSetRecord old(*this);
//     --*this;
//     return old;
// }
   
unsigned long DataSetRecord::count() const{
    return count_;
}

bool DataSetRecord::keep() const{
    return keep_;
}

void DataSetRecord::keep(bool keep){
    keep_ = keep;
}

bool DataSetRecord::active() const{
    return set_ != 0;
}

const string & DataSetRecord::name() const {
    return name_;
}


