// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.




#ifndef __STORAGE_H
#define __STORAGE_H

#include "HiLowDataStorage.h"
#include "DalChild.h"
#include "HiLowColumn.h"
#include "HiLowArray.h"
#include "Tracer.h"

/*AI gcc 4 porting 
parent()->dumpData(...)
this->parent()->dumpData(); OR
DalData<T>::parent() -> dumpData();
*/
template<class T>
class Storage : public HiLowDataStorage, public DalChild<T>
{
	public :
		Storage(
			const std::string&                   name,
                        DataType                        dataType,
                        unsigned long                   size,
			T *				parent )
		: HiLowDataStorage( name, dataType, size ), DalChild<T>(  parent )
		{
			PROC
			ENDPROC
		}

		virtual ~Storage() {
		  PROC
		  // ********************************************
		  // Since the C++ rule is that derived classes
		  // are destroyed before the base class, then
		  // we must release all the data here, since the
		  // load/dump data methods are contained in this
		  // derived class.
		  // ********************************************
		  release();
		  ENDPROC
		}

		void dumpData( char ** values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( bool8 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( int8 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( uint16 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( int16 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( uint32 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( int32 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( real32 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void dumpData( real64 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> dumpData( values, from, count ); ENDPROC }
		void loadData( char ** values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( bool8 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( int8 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( uint16 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( int16 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( uint32 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( int32 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( real32 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void loadData( real64 * values, unsigned long from, unsigned long count ) const { PROC this->parent() -> loadData( values, from, count ); ENDPROC }
		void copyData( unsigned long from, unsigned long to, unsigned long count ) const { PROC this->parent() -> copyData( from, to, count ); ENDPROC }

};


#endif

