// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
/*************************************************************** 
* 
* MemCellData.cc            & XMM Science Analysis System (SAS) 
* 
* Class representing the Data in a Column Cell. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: MemCellData.cc,v 1.2 2004/03/05 14:43:03 eojero Exp $ 
* 
*****************************************************************/ 

#include "errstr.h"
#include "Tracer.h"
#include "vectorutil.h"
#include "MemCellData.h"


MemCellData::MemCellData(
			DataStorage *			data,
			const DalColumn *		parent,
			unsigned long			rowNumber,
			unsigned long			from,
			unsigned long			count,
			AccessMode			accessMode )
		:	DalCellData( parent, rowNumber, from, count ),
			MemData<DalColumn>( data, 0, 0, parent, accessMode )
{
	PROC

	seek( from, count );

	ENDPROC
}


MemCellData::~MemCellData()
{
	PROC

	DalColumn* column = this -> DalChild<DalColumn>::parent();
	column -> removeDataObject( this );

	ENDPROC
}


void
MemCellData::size( unsigned long size )
{
	PROC

	dataStorage() -> size( size ); 
	seek( 0, size );

	ENDPROC
}


unsigned long
MemCellData::size() const
{
	PROC
	RETURN dataStorage() -> size();
}


unsigned long
MemCellData::row() const
{
	PROC
	RETURN _rowNumber;
}


unsigned long
MemCellData::elements() const
{
	PROC
	RETURN dataStorage() -> size();
}


void
MemCellData::seek(
	unsigned long	from,
	unsigned long	count )
{
	PROC

	unsigned long	_from;
	unsigned long	_count;

	_from = from;

	if (count == 0)
		_count = dataStorage() -> size();

	else
		_count = count;

	DalSeekable< unsigned long >::seek( _from, _count );
	// ************************************
	// TODO: Implement properly.
	// In this version seek the entire row.
	// ************************************
	//DalSeekable< unsigned long >::seek(
	//			_rowNumber,
	//			dataStorage() -> size() );

	seekable().seek(
		DalSeekable<unsigned long>::from(),
		DalSeekable<unsigned long>::count() );
	ENDPROC
}







