// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* HiLowFitsReaderWriter.h     & XMM Science Analysis System (SAS) 
* 
* Class representing FITS file reader/writer.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowFitsReaderWriter.h,v 1.2 2004/03/05 14:43:03 eojero Exp $
* 
*****************************************************************/ 
#ifndef HILOW_FITS_READER_WRITER_H
#define HILOW_FITS_READER_WRITER_H


#include "FitsReaderWriter.h"
#include "HiLowFitsDatabase.h"


class HiLowFitsReaderWriter : public virtual FitsReaderWriter, public virtual HiLowFitsDatabase
{
	public :

		HiLowFitsReaderWriter();
	virtual ~HiLowFitsReaderWriter() {}
		void read( DataSet* dataSet );
		void write( DataSet* dataSet );

	private :

		int writeColumns(
			fitsfile *	fptr,
			const Table &	table,
			int &		status ) const;

		int writeTable( 
			fitsfile *	fptr,
			const Table &	table,
			int &		status ) const;

		//int readColumn(
		//	fitsfile *	fptr,
		//	int &		columnNumber,
		//	Table &		table,
		//	int &		status ) const;

		int readTable(
			fitsfile * 	fptr,
			DataSet &	dataSet,
			int &		status ) const;

		int readArray(
			fitsfile *	fptr,
			DataSet &	dataSet,
			int &		status ) const;

		int readPrimaryArray(
			fitsfile *		fptr,
			DataSet &		dataSet,
			KeywordChecker *	keywordChecker,
			int &			status ) const;

		int writeArray(
			fitsfile *	fptr,
			const Array &	array,
			int &		status ) const;

		int deleteProcessHistory(
			fitsfile *	fptr,
			int &		status ) const;

		int deleteUnwantedKeyword(
			fitsfile *	fptr,
			const std::string&	keyword,
			int &		status ) const;

		int deleteAttributes(
			fitsfile *	fptr,
			int &		status ) const;
		//int deletePrimaryHeaderAttributes(
		//	fitsfile *	fptr,
		//	int &		status ) const;

		//int deleteCommentsAndHistory(
		//	fitsfile *		fptr,
		//	KeywordChecker *	attributeChecker,
		//	int &			status ) const;
		int deleteCommentsAndHistory(
			fitsfile *		fptr,
			int &			status ) const;
		int deleteHistory(
			fitsfile *		fptr,
			int &			status ) const;

};


#endif
