// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;

#include <Dal.h>
#include <errstr.h>
#include <sstream>


unsigned int b = 0;
bool first = true;

struct BlockDesc
{
	Block::Type	type;
	unsigned int 	blockNumber; } blockDesc[] =
{
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
 	{ Block::TableT, (first ? (first = false, b ) : ( first = false, ++b )) },
 	{ Block::TableT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::TableT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::TableT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::TableT, (first ? (first = false, b ) : ( first = false, ++b )) },
 	{ Block::TableT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
  	{ Block::ArrayT, (first ? (first = false, b ) : ( first = false, ++b )) },
};


const unsigned int numberOfBlocks = sizeof( blockDesc ) / sizeof( blockDesc[ 0 ] );

static void
test( DataSet * set, unsigned int offset )
{
	// All blocks in one pass, before closing set.
	for( unsigned int i = 0; i < numberOfBlocks; ++i )
	{
		Block *		block = set -> block( i + offset );
		ostringstream	name;

		switch( blockDesc[ i ].type )
		{
			case Block::TableT :
				name << "table" << i;
				break;

			case Block::ArrayT :
				name << "array" << i;
				break;
			default :
				errstr << "Invalid  block type" << Sas::error( "internalError" );
				break;
		} // switch
		// Check block name
		// cout << block -> name() <<  " == " << name.str() << endl;
		if( !( block -> name() == name.str() )) errstr << "invalid block name" << Sas::error( "internalError" );
		// Check block name using position within set
		if( !( set -> block( blockDesc[ i ].blockNumber  + offset) -> name() == name.str() )) errstr << "wrong block name" << Sas::error( "internalError" );
		// Check block number: first method
		if( set -> blockNumber( block -> name() ) != blockDesc[ i ].blockNumber  + offset ) errstr << "wrong block number for table" << Sas::error( "internalError" );
		// Check block number: second method
		if( block -> parent() -> blockNumber( block -> name() ) != blockDesc[ i ].blockNumber  + offset) errstr << "wrong block number for table" << Sas::error( "internalError" );
	} // for
}


int main()
{

	string		fileName = "test.dat";

  	DataSet *set = dataSetServer->open(fileName,DataSet::Create);

  	unsigned long size[] = { 5,7,3,5,4,6 };

	for( unsigned int i = 0; i < numberOfBlocks; ++i )
	{
		Block *		block = 0;
		ostringstream	name;
		ostringstream	label;
		ostringstream	units;

		switch( blockDesc[ i ].type )
		{
			case Block::TableT :
				name << "table" << i;
				label << "table label" << i;
				block = set -> addTable(
					name.str(),
					100,
					label.str(),
					blockDesc[ i ].blockNumber );
				break;

			case Block::ArrayT :
				name << "array" << i;
				label << "array label" << i;
				units << "array units" << i;
				block = set -> addArray(
					name.str(),
					Array::Int32,
					vector<unsigned long>(size,size+3),
					label.str(),
					units.str(),
					blockDesc[ i ].blockNumber );
				break;
			default :
				errstr << "Invalid  block type" << Sas::error( "internalError" );
				break;
		} // switch

		// Check block name
		if( !( block -> name() == name.str() )) errstr << "invalid block name" << Sas::error( "internalError" );
		// Check block name using position within set
		if( !( set -> block( blockDesc[ i ].blockNumber ) -> name() == name.str() )) errstr << "wrong block name" << Sas::error( "internalError" );
		// Check block number: first method
		if( set -> blockNumber( block -> name() ) != blockDesc[ i ].blockNumber ) errstr << "wrong block number for table" << Sas::error( "internalError" );
		// Check block number: second method
		if( block -> parent() -> blockNumber( block -> name() ) != blockDesc[ i ].blockNumber ) errstr << "wrong block number for table" << Sas::error( "internalError" );

	} // for

	// Test with default blocks, while set is still open.
	test( set, 0 );

	dataSetServer->close( set );

  	set = dataSetServer->open(fileName,DataSet::Modify);
	// Test with default blocks, while set has been reopened
	test( set, 0 );
	dataSetServer->close( set );
  	set = dataSetServer->open(fileName,DataSet::Modify);
	set -> addTable(
		"tab0",
		100,
		"",
		0 );
	// Test again, while set is still open
	test( set, 1 );

	dataSetServer->close( set );

  	set = dataSetServer->open(fileName,DataSet::Modify);
	// Test again, while set has been reopened
	test( set, 1 );

	dataSetServer->close( set );

  	set = dataSetServer->open(fileName,DataSet::Modify);
	set -> addTable(
		"tab1",
		100,
		"",
		0 );
	set -> addTable(
		"tab2",
		100,
		"",
		0 );
	set -> addTable(
		"tab3",
		100,
		"",
		0 );
	test( set, 4 );
	dataSetServer->close( set );

	// Now add the primary array.
  	set = dataSetServer->open(fileName,DataSet::Modify);
	set -> addArray(
			"PRIMARY",
			Array::Int32,
			vector<unsigned long>(size,size+3),
			"",
			"",
			0 );
	test( set, 5 );

	dataSetServer->close( set );

  	set = dataSetServer->open(fileName,DataSet::Modify);
	test( set, 5 );
	dataSetServer->close( set );

	return 0;

}
