// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include<cassert>


#include <Dal.h>

int main()
{

  DataSet* set;
  set = dataSetServer->open("test.dat",DataSet::Create);
  *set->addAttribute("att1","set int comment","int unit") = 1;
  vector<unsigned long> size;
  size.push_back(1);
  size.push_back(2);
  size.push_back(3);
  Array* arr = set->addArray("PRIMARY", Array::Int32, size);
  *arr->addAttribute("att2","arr int comment","int unit") = 4;
  arr = set->addArray("array1", Array::Int32, size);
  *arr->addAttribute("att1","arr int comment","int unit") = 4;
  Table* tab = set->addTable("table1",10);
  *tab->addAttribute("att1","tab bool comment") = false;
  Column* col1 = tab-> addColumn("col1", Column::Int32 );
  *col1->addAttribute("tlmin","minimum","int unit") = 1;
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify );
  unsigned int nAttributes = set-> attributes();
  set -> deleteAttribute( "att1" );
  assert( set -> attributes() == nAttributes - 1 );
  arr = set -> array( "PRIMARY" );
  arr -> deleteAttribute( "att2" );
  assert( arr -> attributes() == 0 );
  arr = set -> array( "array1" );
  arr -> deleteAttribute( "att1" );
  assert( arr -> attributes() == 0 );
  tab = set -> table( "table1" );
  tab -> deleteAttribute( "att1" );
  assert( tab -> attributes() == 0 );
  col1 = tab -> column( "col1" );
  col1 -> deleteAttribute( "tlmin" );
  assert( col1 -> attributes() == 0 );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Read );
  assert( !set -> hasAttribute( "att1" ) );
  arr = set -> array( "PRIMARY" );
  assert( !arr -> hasAttribute( "att2" ) );
  assert( arr -> attributes() == 0 );
  arr = set -> array( "array1" );
  assert( !arr -> hasAttribute( "att1" ) );
  assert( arr -> attributes() == 0 );
  tab = set -> table( "table1" );
  assert( !tab -> hasAttribute( "att1" ) );
  assert( tab -> attributes() == 0 );
  col1 = tab -> column( "col1" );
  assert( !col1 -> hasAttribute( "tlmin" ) );
  assert( col1 -> attributes() == 0 );
  dataSetServer->close(set);

  return 0;
}
