// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include <Dal.h>
#include<cassert>

int main()
{
  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  assert( set -> blocks() == 0 );
  vector<unsigned long> size;
  size.push_back(3);
  size.push_back(4);
  set->addArray("PRIMARY", Array::Int8, size );
  assert( set -> blocks() == 1 );
  set->addTable("table1",100);
  assert( set -> blocks() == 2 );
  set->addArray("array2", Array::Int8, size );
  assert( set -> blocks() == 3 );
  set->addArray("array3", Array::Int8, size );
  assert( set -> blocks() == 4 );
  set->addTable("table2",100);
  assert( set -> blocks() == 5 );
  set->addTable("table3",100);
  assert( set -> blocks() == 6 );
  dataSetServer->close(set);


  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( set -> blocks() == 6 );
  set -> deleteBlock( "table1" );
  assert( set -> blocks() == 5 );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( set -> blocks() == 5 );
  set -> deleteBlock( "array2" );
  assert( set -> blocks() == 4 );
  set -> deleteBlock( "table3" );
  assert( set -> blocks() == 3 );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( set -> blocks() == 3 );
  set -> deleteBlock( "table2" );
  assert( set -> blocks() == 2 );
  set -> deleteBlock( "array3" );
  assert( set -> blocks() == 1 );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( set -> blocks() == 1 );
  set -> deleteBlock( "PRIMARY" );
  assert( set -> blocks() == 0 );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Read);
  assert( set -> blocks() == 0 );
  dataSetServer->close(set);

  return 0;
}

