// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include<cassert>

#include <Dal.h>
#include <errstr.h>
#include "StopWatch.h"

void
test()
{

  //char* dummy = new char[ 15000000 ];
  //delete[] dummy;

  DataSet *set = dataSetServer->open("test.dat",DataSet::Create);

  unsigned long size[] = { 4096,4096,1 };

  Array* arr;
  Data* data;
  real64* r64;

  //char* dummy = new char[ 15000000 ];
  //delete[] dummy;
  arr = set->addArray("real64Array", Array::Real64,
	vector<unsigned long>(size,size+3));
  data = arr->data();
  r64 = data->real64Data();
  for (unsigned i = 0; i < data->elements(); ++i)
   r64[i] = 0.25*i;
  delete data;

  char* dummy = new char[ 15000000 ];
  delete[] dummy;

  dataSetServer->close(set);


  set = dataSetServer->open("test.dat",DataSet::Read);
  arr = set->array("real64Array");
  assert( arr->dataType() == Array::Real64 );
  assert( arr->size() == vector<unsigned long>(size,size+3) );
  assert( arr->elements() == size[0]*size[1]*size[2] );
  data = arr->data();
  r64 = data->real64Data();
  for (unsigned i = 0; i < data->elements(); ++i)
   assert( r64[i] == 0.25*i );
  delete data;

  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify);
  arr = set->array("real64Array");
  assert( arr->dataType() == Array::Real64 );
  assert( arr->size() == vector<unsigned long>(size,size+3) );
  assert( arr->elements() == size[0]*size[1]*size[2] );
  data = arr->data();
  r64 = data->real64Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    assert( r64[i] == 0.25*i );
  delete data;

  dataSetServer->close(set);

}


int
main()
{
	// *********************************
	// Get clock for timing information.
	// *********************************
	StopWatch		cpuWatch( StopWatch::CPU );
	StopWatch		calenderWatch( StopWatch::Calender );


	for( unsigned int i = 0; i < 10; ++i )
	{
		test();
		cout << "[" << i << "] CPU time: " << cpuWatch.elapsed() << " secs" << endl;
		cout << "[" << i << "] Calender time: " << calenderWatch.elapsed() << " secs" << endl;
	} // for
	return 0;
}

