// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include <Dal.h>

#include <cassert>


/*
  JBa: gcc-3.1 is very strict!

  choosing
    `virtual const std::string& DataSetServer::process(const DataSet*, unsigned int) const'
   over
    `virtual void DataSetServer::process(const DataSet*, const std::string&)'
   because worst conversion for the former is better than worst conversion for the latter

 */
static unsigned int ProcId(unsigned int i) {return i;}

int main()
{
  dataSetServer->client("dal test harness");


  // retunr an empty string if no process has been set
  assert( dataSetServer -> process().empty() );

  // new dataset, no process
  DataSet *set = dataSetServer->open("test.dat",DataSet::Create);
  assert( dataSetServer -> processes( set ) == 0 );
  dataSetServer->close(set);

  // modify the dataset, still no process
  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( dataSetServer -> processes( set ) == 0 );
  dataSetServer->close(set);

  // modify the dataset 	
  set = dataSetServer->open("test.dat",DataSet::Modify);

  // still no process
  assert( dataSetServer -> processes( set ) == 0 );

  // set the process ...
  dataSetServer -> process( "process0" );
  assert( !dataSetServer -> process().empty() );
  dataSetServer -> process() == "process0";
  // ... but the dataset has no process because it was opened earlier ...
  assert( dataSetServer -> processes( set ) == 0 );
  // ... but when we close the set we expect process to be written
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( dataSetServer -> processes( set ) == 1 );
  assert( dataSetServer -> process( set, ProcId(0) ) == "process0" );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( dataSetServer -> processes( set ) == 2 );
  assert( dataSetServer -> process( set, ProcId(0) ) ==  "process0" );
  assert( dataSetServer -> process( set, ProcId(1) ) ==  "process0" );
  dataSetServer->close(set);

  dataSetServer -> process( "process1" );
  set = dataSetServer->open("test.dat",DataSet::Modify);
  assert( dataSetServer -> processes( set ) == 3 );
  assert( dataSetServer -> process( set, ProcId(0) ) ==  "process0" );
  assert( dataSetServer -> process( set, ProcId(1) ) ==  "process0" );
  assert( dataSetServer -> process( set, ProcId(2) ) ==  "process0" );
  dataSetServer -> process( set, "process2" );
  dataSetServer->close(set);

  dataSetServer -> process( "process3" );
  set = dataSetServer->open("test.dat",DataSet::Modify );
  assert( dataSetServer -> processes( set ) == 5 );
  assert( dataSetServer -> process( set, ProcId(0) ) == "process0" );
  assert( dataSetServer -> process( set, ProcId(1) ) == "process0" );
  assert( dataSetServer -> process( set, ProcId(2) ) == "process0" );
  assert( dataSetServer -> process( set, ProcId(3) ) == "process2" );
  assert( dataSetServer -> process( set, ProcId(4) ) == "process1" );
  dataSetServer -> process( set, "process4" );
  dataSetServer -> process( set, "process5" );
  dataSetServer->close(set);

  set = dataSetServer->open("test.dat",DataSet::Read );
  assert( dataSetServer -> processes( set ) == 8 );
  assert( dataSetServer -> process( set, ProcId(0) ) == "process0" );
  assert( dataSetServer -> process( set, ProcId(1) ) == "process0" );
  assert( dataSetServer -> process( set, ProcId(2) ) == "process0" );
  assert( dataSetServer -> process( set, ProcId(3) ) == "process2" );
  assert( dataSetServer -> process( set, ProcId(4) ) == "process1" );
  assert( dataSetServer -> process( set, ProcId(5) ) == "process4" );
  assert( dataSetServer -> process( set, ProcId(6) ) == "process5" );
  assert( dataSetServer -> process( set, ProcId(7) ) == "process3" );
  dataSetServer->close(set);

  return 0;
}
