// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;

#include <unistd.h>
#include "Tracer.h"
#include "Dal.h"
#include "fitsio.h"
#include "stringutil.h"

static void
testFITS(
	unsigned long	nrows, 
	unsigned long 	nelements )
{
 PROC
   
  try {
    cout << "FITS VERSION" << endl;
    fitsfile*	fptr;
    int		status;
    
    unlink( "test.dat" );
    fits_create_file( &fptr, "test.dat", &status );
    fits_create_img( fptr, 8, 0, 0, &status );
    
    fits_insert_btbl(
		     fptr,
		     nrows,
		     0,
		     (char **)(0),
		     (char **)(0),
		     (char **)(0),
		     "",
		     0,
		     &status );
    /* AI gcc4 porting
     *  deprecated conversion from string constant to char*
     * cast to char*
     */
    fits_insert_col(
		    fptr,
		    1,
		    (char*) "VARCOLUMN",
		    //"1PJ(1)",
		    (char*) "1PJ",
		    //"1PV",
		    &status );
    
    //fits_update_key_str(
    //		fptr,
    //		(char *)( "TFORM1" ),
    //		(char*)(("1PV(" + toString( nelements) + ")").c_str()),
    //		0,
    //		&status );
    
    for( unsigned int i = 0; i < nrows; ++i )
      {
	int32 data[ nelements ];
	fits_write_col_int(
			   fptr,
			   1,
			   i + 1,
			   1,
			   nelements,
			   data,
			   &status );
      } // for
    
    fits_insert_btbl(
		     fptr,
		     nrows,
		     0,
		     (char **)(0),
		     (char **)(0),
		     (char **)(0),
		     "",
		     0,
		     &status );
    /* AI gcc4 porting
     *  deprecated conversion from string constant to char*
     * cast to char*
     */
    fits_insert_col(
		    fptr,
		    1,
		    (char*) "COLUMN",
		    (char*) "E",
		    &status );
    
    fits_close_file( fptr, &status );
    fits_report_error( stderr, status );
    ENDPROC
      }
  catch (...) {
     std::cerr << "DAL failure in varlengthcolumn.cc test "  << std::endl;
  }

}


static void
testDAL(
	unsigned long	nrows, 
	unsigned long 	nelements )
{
  PROC
    
    try {
      cout << "DAL VERSION" << endl;
      DataSet * set = dataSetServer->open("test.dat", DataSet::Create);
      
      Table * table = set->addTable("FIRST", nrows);
      
      Column * col = table->addColumn("VARCOLUMN", 
				      //Column::Uint32, 
				      Column::Int32, 
				      "No comment", 
				      "", 
				      Dal::zero);
      
      for (unsigned int row = 0; row < nrows; ++row)
	{
	  CellData * cellData = col->cellData(row);
	  cellData->size(nelements);
	  
	  //real32 * data = cellData->real32Data();
	  
	  //for (unsigned int element = 0; 
	  //	element < nelements; 
	  //	++element)
	  //{
	  //	data[element] = element;
	  //}
	  
	  delete cellData;
	}
      
      Table * table2 = set->addTable("SECOND", nrows);
      
      table2->addColumn("COLUMN", 
			Column::Real32, 
			"No comment", 
			"");
      
      dataSetServer->close(set);
      ENDPROC
    }
    catch (...) {
    std::cerr << "DAL failure in varlengthcolumn.cc test "  << std::endl;
  }

}


int
main()
{
	PROC
	const unsigned long 	MaxRows = 30;
	const unsigned long 	MaxElements = 25;

	typedef pair<unsigned long,unsigned long>	P;
	vector<P> 		skips;
	/* AI gcc4 porting
	 * testDAL and testFITS do not return anything, so cannot be compared in a
	 * id statment.
	 * try catch statment added in the corresponding subroutines
	if( testDAL == 0 );
	if( testFITS == 0 );
	*/

	if( 0 )
	{
	skips.push_back( P( 13,27 ) );
	skips.push_back( P( 13,28 ) );
	skips.push_back( P( 13,29 ) );
	skips.push_back( P( 14,26 ) );
	skips.push_back( P( 14,25 ) );
	skips.push_back( P( 19,18 ) );
	skips.push_back( P( 19,19 ) );
	skips.push_back( P( 28,12 ) );
	skips.push_back( P( 20,19 ) );
	skips.push_back( P( 21,18 ) );
	skips.push_back( P( 21,19 ) );
	skips.push_back( P( 22,17 ) );
	}
	const unsigned long	bRow = 0;

	for( unsigned long i = bRow; i < MaxRows; ++i )
	{
		for( unsigned long j = 0; j < MaxElements; ++j )
		{
			bool skipped = false;

			cout << "[" << i << "," << j << "]"; 

			for( unsigned long m = 0; m < skips.size(); ++m )
			{
				if( skips[ m ].first == i && skips[ m ].second == j )
				{
					cout << " skipped: product = " << (skips[ m ].first+1) * (skips[ m ].second+1) << endl; 
					skipped = true;
					break;
				} // if
			} // for
			if( !skipped )
			{
				testFITS( i, j );
				testDAL( i, j );
			} // if
			cout << endl;
		} // for
	} // for
	ENDPROC
}
