// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include <ExtDal.h>
#include <sstream>
#include <Tracer.h>
#include <StopWatch.h>
#include<cassert>

int
main()
{
	// *********************************
	// Get clock for timing information.
	// *********************************
	StopWatch		cpuWatch( StopWatch::CPU );
	StopWatch		calenderWatch( StopWatch::Calender );

  	unsigned long nRows = 10000;

	DataSet*		set;
	Table*			tab;
	Array*			arr;
	vector<unsigned long>	size;

	size.push_back( 5 );

  	set = dataSetServer->open("test.dat",DataSet::Create);
  	tab = set->addTable("table1",nRows);
  	arr = set->addArray("array1",Array::Int32,size );
  	tab = set->addTable("table2",nRows);
  	arr = set->addArray("array2",Array::Int32,size );
  	tab = set->addTable("table3",nRows);
  	arr = set->addArray("array3",Array::Int32,size );
  	tab = set->addTable("table4",nRows);
  	arr = set->addArray("array4",Array::Int32,size );
  	tab = set->addTable("table5",nRows);
  	arr = set->addArray("array5",Array::Int32,size );
  	dataSetServer->close(set);


	set = dataSetServer->open("test.dat",DataSet::Read);

	TableIterator* ti = blockIteratorServer -> tableIterator( set );
	ArrayIterator* ai = blockIteratorServer -> arrayIterator( set );

	unsigned int i = 0;

	while( (tab = ti -> next()) && (arr = ai -> next() ))
	{
		assert( tab -> blockNumber() == 2 * i );
		assert( arr -> blockNumber() == 2 * i + 1 );
		++i;
	} // while

	delete ti;
	delete ai;

  	dataSetServer->close(set);

	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

  	return 0;
}

