! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

subroutine fill( set, nRows )

  use dal

  implicit none
  
  type(DataSetT) set
  integer nRows
  type(TableT) tab
  type(ColumnT) col
  real(kind=single), dimension(:), pointer :: c
  integer m

  tab = addTable(set, "table", nRows, "table comment" )
  col = addColumn( tab, "column1", REAL32, "km", (/ 1 /), "column comment" )

  do m=0,numberOfRows(tab)-1
     c => real32Data(col,m)
     c = m+3
     if(modulo(m,1000) .eq. 0)then
        write(*,*)m
     end if
     call release(col)
  end do
  call release(tab)
end subroutine fill


subroutine check( set )

  use dal
  use errorHandling

  implicit none
  
  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col
  real(kind=single), dimension(:), pointer :: c
  integer m

  tab = table( set, "table" )
  col = column( tab, "column1" )

  do m=0,numberOfRows(tab)-1
     c => real32Data(col,m)
       if( any( c /= m + 3 )) call error( "Test failed" )

     if(modulo(m,1000) .eq. 0)then
        write(*,*)m
     end if
     call release(col)
  end do
  call release(tab)
end subroutine check


subroutine dotest( nRows )

  use dal

  implicit none
  
  type(DataSetT) set
  integer nRows

  set = dataSet("test.dat",CREATE)
  call fill( set, nRows )
  call check( set )
  call release(set)

  set = dataSet("test.dat",READ)
  call check( set )
  call release(set)
end subroutine dotest


program test

  use dal

  implicit none

  call dotest( 10000 )

end program test

