! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program test

  use dal
  use errorHandling

  type(DataSetT) :: set1, set2
  type(TableT) :: tab1, tab2
  type(ArrayT) :: arr1, arr2

  set1 = dataSet("test1.dat",create)
  call setAttribute(set1,"sbool",.false.,"set bool comment")
  call setAttribute(set1,"sint",3,"int unit","set int comment")
  call setAttribute(set1,"sreal",2.5,"real unit","set real comment")
  call setAttribute(set1,"sstring","a set string","set string comment")
  call addComment(set1,"comment 1")
  call addComment(set1,"comment 2")
  call addHistory(set1,"history 1")
  call addHistory(set1,"history 2")

  arr1 = addArray(set1,"array",integer32, dimensions=(/ 1,2,3 /))
  call setAttribute(arr1,"abool",.true.,"arr bool comment")
  call setAttribute(arr1,"aint",4,"int unit","arr int comment")
  call setAttribute(arr1,"areal",3.5,"real unit","arr real comment")
  call setAttribute(arr1,"astring","an array string","arr string comment")
  call release(arr1)

  tab1 = addTable(set1,"table",10)
  call setAttribute(tab1,"abool",.false.,"tab bool comment")
  call setAttribute(tab1,"aint",7,"int unit","tab int comment")
  call setAttribute(tab1,"areal",5.5,"real unit","tab real comment")
  call setAttribute(tab1,"astring","a table string","tab string comment")
  call release(tab1)


  set2 = dataSet("test2.dat",create)
  call setAttributes(attributable(set2),attributable(set1))

  if ( booleanAttribute(set2,"sbool") ) &
	call error("testFailed","set bool value")

  if ( int32Attribute(set2,"sint") /= 3 ) &
	call error("testFailed","set int value")

  if ( real64Attribute(set2,"sreal") /= 2.5 ) &
	call error("testFailed","set real value")

  if ( stringAttribute(set2,"sstring") /= "a set string" ) &
	call error("testFailed","set string value")

  arr2 = addArray(set2,"array",integer32, dimensions=(/ 1,2,3 /))
  call setAttributes(attributable(arr2),attributable(arr1))

  if ( .not. booleanAttribute(arr2,"abool") ) &
	call error("testFailed","arr bool value")

  if ( int32Attribute(arr2,"aint") /= 4 ) &
	call error("testFailed","arr int value")

  if ( real64Attribute(arr2,"areal") /= 3.5 ) &
	call error("testFailed","arr real value")

  if ( stringAttribute(arr2,"astring") /= "an array string" ) &
	call error("testFailed","arr string value")

  call release(arr2)

  tab2 = addTable(set2,"table",100)
  call setAttributes(attributable(tab2),attributable(tab1))

  if ( booleanAttribute(tab2,"abool") ) &
	call error("testFailed","tab bool value")

  if ( int32Attribute(tab2,"aint") /= 7 ) &
	call error("testFailed","tab int value")

  if ( real64Attribute(tab2,"areal") /= 5.5 ) &
	call error("testFailed","tab real value")

  if ( stringAttribute(tab2,"astring") /= "a table string" ) &
	call error("testFailed","set string value")

  call release(tab2)

  call release(set1)
  call release(set2)

end program test
