! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

subroutine check( l, thisName, thisLabel )
  use dal
  use errorhandling

  implicit none

  type(LabelledT), intent(in) :: l
  character(len=*), intent(in) :: thisName
  character(len=*), intent(in) :: thisLabel

  write(*,*) name(l), label(l), thisName, thisLabel

  if( name( l ) /= thisName ) then
    write(errstr,*) "Incorrect object name. Actual: ", name(l), " Expected: ", thisName
    call error( "internalError" )
  endif

  if( label(l) /= thisLabel ) then
    write(errstr,*) "Incorrect object label. Actual: ", label(l), " Expected: ", thisLabel
    call error( "internalError" )
  endif

end subroutine check


program test

  use dal

  implicit none

  type(DataSetT) set
  type(ArrayT) arr
  type(TableT) tab
  type(ColumnT) col
  ! type(AttributeT) att
  ! integer(kind=int32), dimension(:,:,:), pointer :: a
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)

  ! create a set
  set = dataSet("test.dat",CREATE)
  call setAttribute(set,"att1","value1","a dataset attribute comment")
  arr = addArray(set, "array", INTEGER32, comment="an array comment", dimensions=s )
  call setAttribute(arr,"att2","value2","an array attribute comment")
  tab = addTable(set, "table", 10, comment="a table comment" )
  call setAttribute(tab,"att3","value3","a table attribute comment")
  col = addColumn(tab,"int8",INTEGER8,comment="a column comment")
  call setAttribute(col,"TLMAX","value4","a column attribute comment")
   
  call check( labelled(set), "test.dat", "" )
  call check( labelled(arr), "array", "an array comment" )
  call check( labelled(tab), "table", "a table comment" )
  call relabel( tab, "a new table comment" )
  call check( labelled(tab), "table", "a new table comment" )

  call release( set )

end program test
