! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

subroutine fill( nrows )

  use dal
  use errorhandling
  
  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col
  integer(kind=int8), dimension(:), pointer :: i8
  integer(kind=int16), dimension(:), pointer :: i16
  integer(kind=int32), dimension(:), pointer :: i32
  real(kind=single), dimension(:), pointer :: r32
  real(kind=double), dimension(:), pointer :: r64
  integer i
  integer nrows

  set = dataSet("test.dat",CREATE)
  tab = addTable(set,"some table",nrows)

  col = addColumn(tab,"int8",INTEGER8,units="cm",comment="int8 column")

  if( nullDefined( col ) ) then
    write(errstr,*) "The null value should not have been defined."
    call error( "internalError" )
  endif
  
  call setNullValue( col, 0 )

  if( .not.nullDefined( col ) ) then
    write(errstr,*) "The null value should have been defined."
    call error( "internalError" )
  endif

  i8 => int8Data(col)
  write(*,*) shape(i8)
  do i=0,numberOfRows(tab)-1
    i8(i) = i
  end do
  call settonull( col, 0 )
  call settonull( col, nRows - 1 )

  col = addColumn(tab,"int16",INTEGER16,units="dm",comment="int16 column")

  if( nullDefined( col ) ) then
    write(errstr,*) "The null value should not have been defined."
    call error( "internalError" )
  endif

  call setNullValue( col, 0 )

  if( .not.nullDefined( col ) ) then
    write(errstr,*) "The null value should have been defined."
    call error( "internalError" )
  endif

  i16 => int16Data(col)
  do i=0,numberOfRows(tab)-1
    i16(i) = 2*i
  end do
  call settonull( col, 0 )
  call settonull( col, nRows - 1 )

  col = addColumn(tab,"int32",INTEGER32,units="m",comment="in32 column")

  if( nullDefined( col ) ) then
    write(errstr,*) "The null value should not have been defined."
    call error( "internalError" )
  endif

  call setNullValue( col, 0 )

  if( .not.nullDefined( col ) ) then
    write(errstr,*) "The null value should have been defined."
    call error( "internalError" )
  endif

  i32 => int32Data(col)
  do i=0,numberOfRows(tab)-1
    i32(i) = 3*i
  end do
  call settonull( col, 0 )
  call settonull( col, nRows - 1 )

  col = addColumn(tab,"real32",REAL32,units="Dm",comment="real32 column")
  r32 => real32Data(col)
  do i=0,numberOfRows(tab)-1
    r32(i) = 0.5*i
  end do
  call settonull( col, 0 )
  call settonull( col, nRows - 1 )

  col = addColumn(tab,"real64",REAL64,units="hm",comment="real64 column")
  r64 => real64Data(col)
  do i=0,numberOfRows(tab)-1
    r64(i) = 0.25*i
  end do
  call settonull( col, 0 )
  call settonull( col, nRows - 1 )

  call release(set)

end subroutine fill


subroutine check( nrows )

  use dal
  use errorhandling
  
  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col
  integer(kind=int8), dimension(:), pointer :: i8
  integer(kind=int16), dimension(:), pointer :: i16
  integer(kind=int32), dimension(:), pointer :: i32
  real(kind=single), dimension(:), pointer :: r32
  real(kind=double), dimension(:), pointer :: r64
  integer i
  integer nrows

  set = dataSet("test.dat",READ)
  tab = table(set,"some table")

  col = column(tab,"int8",READ)

  if( .not.nullDefined( col ) ) then
    write(errstr,*) "The null value should have been defined."
    call error( "internalError" )
  endif

  if( intNullValue( col ) /= 0 ) then
    write(errstr,*) "The column's null value is incorrect in column", name( col)
    call error( "internalError" )
  endif
  i8 => int8Data(col)
  do i=0,numberOfRows(tab)-1
    if( .not.isNull( col,i )) then
      if( i8(i) /= i ) then
        write(errstr,*) "data in column is not correct in column", name( col )
        call error( "internalError" )
      endif
    endif
  end do
  if( .not.isNull( col, 0 ).or. .not.isNull( col, nRows - 1 ) ) then
    write(errstr,*) "Value was expected to be null in column", name( col )
    call error("internalError")
  endif

  col = column(tab,"int16",READ)

  if( .not.nullDefined( col ) ) then
    write(errstr,*) "The null value should have been defined."
    call error( "internalError" )
  endif

  if( intNullValue( col ) /= 0 ) then
    write(errstr,*) "The column's null value is incorrect in column", name( col)
    call error( "internalError" )
  endif
  i16 => int16Data(col)
  do i=0,numberOfRows(tab)-1
    if( .not.isNull( col,i )) then
      if( i16(i) /= 2*i ) then
        write(errstr,*) "data in column is not correct in column", name( col )
        call error( "internalError" )
      endif
    endif
  end do
  if( .not.isNull( col, 0 ).or. .not.isNull( col, nRows - 1 ) ) then
    write(errstr,*) "Value was expected to be null in column", name( col )
    call error("internalError")
  endif

  col = column(tab,"int32",READ)

  if( .not.nullDefined( col ) ) then
    write(errstr,*) "The null value should have been defined."
    call error( "internalError" )
  endif

  if( intNullValue( col ) /= 0 ) then
    write(errstr,*) "The column's null value is incorrect in column", name( col)
    call error( "internalError" )
  endif
  i32 => int32Data(col)
  do i=0,numberOfRows(tab)-1
    if( .not.isNull( col,i )) then
      if( i32(i) /= 3*i ) then
        write(errstr,*) "data in column is not correct in column", name( col )
        call error( "internalError" )
      endif
    endif
  end do
  if( .not.isNull( col, 0 ).or. .not.isNull( col, nRows - 1 ) ) then
    write(errstr,*) "Value was expected to be null in column", name( col )
    call error("internalError")
  endif

  col = column(tab,"real32",READ)
  r32 => real32Data(col)
  do i=1,numberOfRows(tab)-1
    if( .not.isNull( col,i )) then
      if( r32(i) /= 0.5*i ) then
        write(errstr,*) "data in column is not correct in column", name( col )
        call error( "internalError" )
      endif
    endif
  end do
  if( .not.isNull( col, 0 ) ) then
    write(errstr,*) "Value was expected to be null in column", name( col )
    !call error( "internalError" )
  end if

  col = column(tab,"real64",READ)
  r64 => real64Data(col)
  do i=1,numberOfRows(tab)-1
    if( .not.isNull( col,i )) then
      if( r64(i) /= 0.25*i ) then
        write(errstr,*) "data in column is not correct in column", name( col )
        call error( "internalError" )
      endif
    endif
  end do
  if( .not.isNull( col, 0 ) ) then
    write(errstr,*) "Value was expected to be null in column", name( col )
    call error( "internalError" )
  endif

  call release(set)

end subroutine check


program test

	call fill( 10 )
	call check( 10 )
end program test


