! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

program test

  use dal
  use errorHandling

  type(DataSetT) :: set
  type(ArrayT) :: arr
  type(TableT) :: tab
  type(ColumnT) :: col
  type(AttributeT) :: att

  set = dataSet("test.dat",create)
  call setAttribute(set,"sbool",.false.,"set bool comment")

  arr = addArray(set,"array",integer32, dimensions=(/ 1,2,3 /))
  call setAttribute(arr,"abool",.true.,"arr bool comment")

  tab = addTable(set,"table",10)
  call setAttribute(tab,"tbool",.false.,"tab bool comment")

  col = addColumn(tab,"column",INT32,units="UNITS",comment="Column")
  call setAttribute(col,"tlmin",1_int32,"int32 unit","col int32 comment")

  write(*,*) "qualified data set name: ", qualifiedName( set )
  if( qualifiedName( set ) /= "test.dat" ) then
    call error("dalInternalError")
  end if

  att = attribute( set, "sbool" )
  write(*,*) "qualified data set attribute name: ", qualifiedName( att )
  if( qualifiedName( att) /= "test.dat%sbool" ) then
    call error("dalInternalError")
  end if

  write(*,*) "qualified table name: ", qualifiedName( tab )
  if( qualifiedName( tab ) /= "test.dat:table" ) then
    call error("dalInternalError")
  end if

  att = attribute( tab, "tbool" )
  write(*,*) "qualified table attribute name: ", qualifiedName( att )
  if( qualifiedName( att ) /= "test.dat:table%tbool" ) then
    call error("dalInternalError")
  end if

  write(*,*) "qualified array name: ", qualifiedName( arr )
  if( qualifiedName( arr ) /= "test.dat:array" ) then
    call error("dalInternalError")
  end if

  att = attribute( arr, "abool" )
  write(*,*) "qualified array attribute name: ", qualifiedName( att )
  if( qualifiedName( att ) /= "test.dat:array%abool" ) then
    call error("dalInternalError")
  end if

  write(*,*) "qualified column name: ", qualifiedName( col )
  if( qualifiedName( col ) /= "test.dat:table:column" ) then
    call error("dalInternalError")
  end if

  att = attribute( col, "tlmin" )
  write(*,*) "qualified array attribute name: ", qualifiedName( att )
  if( qualifiedName( att ) /= "test.dat:table:column%tlmin" ) then
    call error("dalInternalError")
  end if

  call release(set)

end program test
