// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine science & computing) 
// September 2001
//
#include<Dal.h>

#include<iostream>
#include<string>
#include<cassert>

int main()
{

  // SPR-2159
  // set->add(block) not implemented if block contains variable length columns.
  // Closed in version 1.138


  dataSetServer->client("dal_test");

  const size_t nrows = 100;
  std::string set1 = "spr2159_origin.ds";
  std::string set2 = "spr2159_destination.ds";

  DataSet *s1 = dataSetServer->open(set1, Dal::Create);
  Table *t = s1->addTable("TABLE", nrows);  

  {
    Column * c = t->addColumn("real32",Dal::Real32,"","",Dal::zero);
    for(unsigned long k = 0; k < nrows; ++k){      
      CellData *d = c->cellData(k);
      d->size(k+1);
      real32 * v = d->real32Data();
      for(unsigned j = 0; j < d->size(); ++j)
	v[j] = k + j;    
      delete d;
    }
  }



  {
    Column * c = t->addColumn("real64",Dal::Real64,"","",Dal::zero);
    for(unsigned long k = 0; k < nrows; ++k){      
      CellData *d = c->cellData(k);
      d->size(k+1);
      real64 * v = d->real64Data();
      for(unsigned j = 0; j < d->size(); ++j)
	v[j] = k + j;    
      delete d;
    }
  }

  {
    Column * c = t->addColumn("int32",Dal::Int32,"","",Dal::zero);
    for(unsigned long k = 0; k < nrows; ++k){      
      CellData *d = c->cellData(k);
      d->size(k+1);
      int32 * v = d->int32Data();
      for(unsigned j = 0; j < d->size(); ++j)
	v[j] = k + j;    
      delete d;
    }
  }

  // cfitsio cannot add a vlength unsigned column
  {
    Column * c = t->addColumn("uint32",Dal::Uint32,"","",Dal::zero);
    for(unsigned long k = 0; k < nrows; ++k){      
      CellData *d = c->cellData(k);
      d->size(k+1);
      uint32 * v = d->uint32Data();
      for(unsigned j = 0; j < d->size(); ++j)
	v[j] = k + j;    
      delete d;
    }
  }

  dataSetServer->close(s1);

  s1 = dataSetServer->open(set1, Dal::Read);

  DataSet * s2 = dataSetServer->open(set2, Dal::Create);

  for(unsigned k = 0; k < s1->blocks(); ++k)
    s2->add(s1->block(k));
  
  
  dataSetServer->close(s1);
  dataSetServer->close(s2);

  s2 = dataSetServer->open(set2, Dal::Read);
  assert(s2->table("TABLE")->hasColumn("real32"));
  assert(s2->table("TABLE")->hasColumn("real64"));
  assert(s2->table("TABLE")->hasColumn("int32"));
//  assert(s2->table("TABLE")->hasColumn("uint32"));
  dataSetServer->close(s2);

}
