// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine science & computing) 
//

// Show that SOC-SPR-2390 is not caused by cfitsio.

#include<Dal.h>
#include<iostream>
#include<cassert>

#include<cstdlib>
#include<cstring>
#include<fitsio.h>
#include<longnam.h>

// Run with an argument to make it fail.

int main(int argc, char**)
{
    
    
    {
	const int NROWS = 128;
	const int NELMS = 20;

	double * arr;

	int status = 0;
	fitsfile *fptr;
	/* AI gcc4 portin
	 * deprecated conversion from string constant to char*=> cast to (char*)
	 */
	char * ttype1[] = {(char*) "EVALUE" };
	char * tform1[] = {(char*) "1PE" };
//	char * tunit1[] = {(char*) "s" };
	
	fits_create_file(&fptr, "!cfits.ds", &status);
	printf("%d-", status);

	fits_create_tbl(fptr, BINARY_TBL, NROWS, 0, 0, 0, 0, 0, &status);
	printf("%d-", status);

	fits_insert_col(fptr, 1, ttype1[0], tform1[0], &status);
	printf("%d-", status);

	for(int k = 1; k <= NROWS ; ++k){
        arr = static_cast<double *>(calloc((size_t)NELMS,sizeof(double)));
	    fits_write_col(fptr, TDOUBLE, 1, k, 1, NELMS, arr, &status);
//	    fits_write_col(fptr, TDOUBLE, 1, k, 1, NELMS, arr, &status);
	    free(arr);
	    long pcount = -1;
	    fits_read_key_lng(fptr, "PCOUNT", &pcount, 0, &status);
	    std::cout << "PCOUNT while writing " << pcount << std::endl;
	}

	printf("%d-", status);

	char * ttype2[] = { (char*) "Ivalue" };
	char * tform2[] = { (char*) "I" };
	char * tunit2[] = { (char*) "s" };
	
	// SOC-SPR-2390: the DAL uses insert_btbl, but because it does
	// not know the value to assign to PCOUNT it should use
	// create_tbl instead.
	if(argc > 1)
	    fits_insert_btbl(fptr, NROWS, 0, ttype2, tform2, tunit2,
			     "table", 0, &status);
	else
	    fits_create_tbl(fptr, BINARY_TBL, NROWS, 0, ttype2, tform2, tunit2,
			    "table", &status);
	printf("%d\n", status);

	int type;
	fits_movabs_hdu(fptr, 2, &type, &status);
	printf("%d-", status);	
	long pcount = -1;
	fits_read_key_lng(fptr, "PCOUNT", &pcount, 0, &status);
	printf("%d\n", status);	
	std::cout << "PCOUNT after new table " << pcount << std::endl;
	
	fits_movabs_hdu(fptr, 3, &type, &status);
	printf("%d-", status);	
	fits_insert_col(fptr, 1, ttype2[0], tform2[0], &status);
	printf("%d-", status);		

	fits_close_file(fptr, &status);        
	printf("%d\n", status);

	fits_open_file(&fptr, "cfits.ds", READONLY, &status);
	printf("%d-", status);
	fits_movabs_hdu(fptr, 2, &type, &status);
	printf("%d-", status);	
	pcount = -1;
	fits_read_key_lng(fptr, "PCOUNT", &pcount, 0, &status);
	printf("%d-", status);	
	std::cout << "PCOUNT after re-open " << pcount << std::endl;
	fits_close_file(fptr, &status);        
	printf("%d\n", status);


	
	assert(status == 0);
	
    }
    
    
}
